## ETAS: An R Package for Fitting the Space-time ETAS Model to Earthquake Data 
## Abdollah Jalilian, Razi University

help("DateTimeClasses")

parallel::detectCores()

# =================================================
# Data example: Section 6
# =================================================

# loading the package
library("ETAS")

# an Iranian eqrthquake catalog accompanying the package
summary(iran.quakes)
diff(as.Date(c("1973-01-01", "1991-01-01")))
diff(as.Date(c("1991-01-01", "2011-01-01")))

# Table 1
head(iran.quakes, n = 11)
tail(iran.quakes, n = 4)

# -----------------------------------------------
# data preparation
# -----------------------------------------------

# specifying a polygonal geographical region
greion <- list(lat = c(26, 25, 29, 38, 35), long = c(52, 59, 58, 45, 43))

# creating an earthquake catalog
iran.cat <- catalog(iran.quakes, study.start = "1991/01/01",
 study.end = "2011/01/01", region.poly = greion, mag.threshold = 4.5)

# printing the specified catalog
iran.cat

# ploting the specified catalog
plot(iran.cat)

# -----------------------------------------------
# model fitting
# -----------------------------------------------

# initial parameter values
param01 <- c(0.5, 0.2, 0.05, 2.7, 1.2, 0.02, 2.3, 0.03)

# fit the model
# attention: it can take up to 90 minutes on a standard personal computer
iran.fit <- etas(iran.cat, param0 = param01)

# additional timings; requires that 4 and 8 threads are available
# respectively
iran.fit <- etas(iran.cat, param0 = param01, nthreads = 4)
iran.fit <- etas(iran.cat, param0 = param01, nthreads = 8)

# print estimates of the model parameters
iran.fit

# plot estimates of the model parameters in each iteration
plot(iran.fit)

# estimates of the background seismicity rate and clustering coefficient
rates(iran.fit)

# estimates of the declustering probabilities
pr <- probs(iran.fit)
summary(pr$prob)

# -----------------------------------------------
# stochastic declustering
# -----------------------------------------------

# detecting most likely background and triggered target events
plot(iran.cat$longlat.coord[pr$target & (1 - pr$prob > 0.95), 1:2])
points(iran.cat$longlat.coord[pr$target & (pr$prob > 0.95), 1:2], pch = 3, col = 2)
map("world", add = TRUE, col = "grey")
legend("bottomleft", c("background", "triggered"), pch = c(1, 3), col = 1:2)

# -----------------------------------------------
# model assessment
# -----------------------------------------------

# residuals and diagnostic plots
iran.res <- resid.etas(iran.fit)
summary(iran.res$tres) 
summary(na.omit(iran.res$sres$z)) 
ks.test(iran.res$U, punif)

# =================================================
# comparison with etasFLP package: Section 7.1
# =================================================

library("etasFLP")
data("italycatalog", package = "etasFLP")

italy.fit.flp <- etasclass(italycatalog, magn.threshold = 3,  
    magn.threshold.back = 3, mu=1, k0 = 0.005, c = 0.005, p = 1.01, 
    a = 1.05, gamma = 0.6, q = 1.52, d = 1.1, ndeclust = 11)
summary(italy.fit.flp)

library("ETAS")
italy.cat <- catalog(italy.quakes, dist.unit = "km")
italy.cat
plot(italy.cat)

k0 <- 0.005
c <- 0.005
p <- 1.01
D <- 1.1
q <- 1.52
A <- pi * k0 / ((p - 1) * c^(p - 1) * (q - 1) * D^(q - 1))
param0 <- c(mu = 1, A, c, alpha = 1.05, p, D, q, gamma = 0.6)
italy.fit <- etas(italy.cat, param0)

# additional timings; requires that 4 and 8 threads are available
# respectively
italy.fit <- etas(italy.cat, param0, nthreads = 4)
italy.fit <- etas(italy.cat, param0, nthreads = 8)

italy.fit

italy.res <- resid.etas(italy.fit)
ks.test(italy.res$U, punif)

italy.res.flp <- plot(italy.fit.flp)
tau.flp <- italy.res.flp$t.trasf
U.flp <- 1 - exp(-diff(tau.flp))
ks.test(U.flp, punif)

# =================================================
# comparison with the Fortran program: Section 7.2
# =================================================

jpoly <- list(long = c(134.0, 137.9, 143.1, 144.9, 147.8,
                       137.8, 137.4, 135.1, 130.6),
              lat = c(31.9, 33.0, 33.2, 35.2, 41.3, 
                      44.2, 40.2, 38.0, 35.4))
japan.cat <- catalog(japan.quakes, study.start = "1953-05-26",
     study.end = "1990-01-08", region.poly = jpoly, mag.threshold = 4.5)
japan.cat
plot(japan.cat)

param0 <- c(0.592844590, 0.204288231, 0.022692883, 1.495169224,
            1.109752319, 0.001175925, 1.860044210, 1.041549634)
japan.fit <- etas(japan.cat, param0)
japan.fit

# additional timings; requires that 4 and 8 threads are available
# respectively
japan.fit <- etas(japan.cat, param0, nthreads = 4)
japan.fit <- etas(japan.cat, param0, nthreads = 8)

rates(japan.fit, long.range = c(134, 145), lat.range = c(31.8, 44.3), 
      dimyx = c(401, 401))

japan.res <- resid.etas(japan.fit)
summary(japan.res$tres)

summary(na.omit(japan.res$sres$z))

ks.test(japan.res$U, punif)

# -------------------------------------
# Figure 11
# -------------------------------------

# Estimate of the background seismicity rate for the Japanese catalog
# using the Fortran program etas8p

# Running etas8p as in v88c01.sh generates two output files: rates.dat and probs.dat
# the output file rates.dat contains estimates background seismicity rate

# path to the output file rates.dat
outfile <- "rates.dat"

# the first two lines are the grid dimensions
dim <- scan(outfile, n = 2)

# the rest of the file contains the background seismicity rate (first column), 
# the total spatial rate (second column) and the conditional intensity 
# function (third column) at grid points
jap.fort.rates <- read.table(outfile, skip = 2)

# construct the grid points
xx <- seq(134, 144, length.out = dim[1])
yy <- seq(32, 44, length.out = dim[2])

# extract the background seismicity rate at grid points
z <- matrix(jap.fort.rates[, 1], nrow = dim[1], byrow = TRUE)

# plot
library("fields")
image.plot(list(x = xx, y = yy, z = z))

# -------------------------------------
# comparing the fitted models by etas8p program and ETAS package
# -------------------------------------

# results from the etas8p program: jap.out file
loglike <- -15310.9518614449
param <- c(0.55061, 0.16576, 0.029615, 1.6579, 
           1.1534, 0.0018338, 1.9505, 1.0670)

# maximum absolute difference between the estimates of 
# the model parameters obtained with the etas8p program and ETAS package
max(abs(japan.fit$param - param))

# absolute difference between the values of the log-likelihood function 
# (hence AIC) between the ETAS model fitted by etas8p program and ETAS package
abs(japan.fit$loglikfv[japan.fit$itr] - loglike)
